@@ -1,6 +1,6 @@
 ;;;; chicken-syntax.scm - non-standard syntax extensions
 ;
-; Copyright (c) 2008-2013, The Chicken Team
+; Copyright (c) 2008-2016, The CHICKEN Team
 ; Copyright (c) 2000-2007, Felix L. Winkelmann
 ; All rights reserved.
 ;
@@ -35,9 +35,7 @@
   (no-bound-checks)
   (no-procedure-checks))
 
-(##sys#provide
- 'chicken-more-macros 			; historical, remove later
- 'chicken-syntax)
+(##sys#provide 'chicken-syntax)
 
 
 ;;; Non-standard macros:
@@ -173,17 +171,19 @@
       (##sys#check-syntax 'assert form '#(_ 1))
       (let* ((exp (cadr form))
 	     (msg-and-args (cddr form))
-	     (msg (optional msg-and-args "assertion failed")))
+	     (msg (optional msg-and-args "assertion failed"))
+	     (tmp (r 'tmp)))
 	(when (string? msg)
 	  (and-let* ((ln (get-line-number form)))
 	    (set! msg (string-append "(" ln ") " msg))))
-	`(##core#if (##core#check ,exp)
-		    (##core#undefined)
-		    (##sys#error 
-		     ,msg 
-		     ,@(if (pair? msg-and-args)
-			   (cdr msg-and-args)
-			   `((##core#quote ,(##sys#strip-syntax exp)))))))))))
+	`(##core#let ((,tmp ,exp))
+	   (##core#if (##core#check ,tmp)
+		      ,tmp
+		      (##sys#error
+		       ,msg
+		       ,@(if (pair? msg-and-args)
+			     (cdr msg-and-args)
+			     `((##core#quote ,(##sys#strip-syntax exp))))))))))))
 
 (##sys#extend-macro-environment
  'ensure
@@ -279,30 +279,39 @@
     (##sys#check-syntax 'parameterize form '#(_ 2))
     (let* ((bindings (cadr form))
 	   (body (cddr form))
-	   (swap (r 'swap))
-	   (mode (r 'mode))
+	   (the (r 'the))
+	   (boolean (r 'boolean))
+	   (convert? (r 'convert?))
 	   (params (##sys#map car bindings))
 	   (vals (##sys#map cadr bindings))
-	   (aliases (##sys#map (lambda (z) (r (pname z))) params))
-	   (aliases2 (##sys#map (lambda (z) (r (gensym))) params)) )
+	   (param-aliases (##sys#map (lambda (z) (r (pname z))) params))
+	   (saveds (##sys#map (lambda (z) (r (gensym 'saved))) params))
+	   (temps (##sys#map (lambda (z) (r (gensym 'tmp))) params)) )
       `(##core#let
-	,(map ##sys#list aliases params)
+	,(map ##sys#list param-aliases params) ; These may be expressions
 	(##core#let
-	 ,(map ##sys#list aliases2 vals)
+	 ,(map ##sys#list saveds vals)
 	 (##core#let
-	  ((,mode #f))
-	  (##core#let
-	   ((,swap (##core#lambda
-		    ()
-		    ,@(map (lambda (a a2)
-			     `(##core#let ((t (,a))) (,a ,a2 ,mode)
-					  (##core#set! ,a2 t)))
-			   aliases aliases2)
-		    (##core#set! ,mode #t))))
-	   (##sys#dynamic-wind 
-	    ,swap
-	    (##core#lambda () ,@body)
-	    ,swap) ) ) ) )))))
+	  ((,convert? (,the ,boolean #t))) ; Convert only first time extent is entered!
+	  (##sys#dynamic-wind
+	   (##core#lambda ()
+	    (##core#let
+	     ;; First, call converters (which may throw exceptions!)
+	     ,(map (lambda (p s temp)
+		     `(,temp (##core#if ,convert? (,p ,s #t #f) ,s)))
+		   param-aliases saveds temps)
+	     ;; Save current values so we can restore them later
+	     ,@(map (lambda (p s) `(##core#set! ,s (,p)))
+		    param-aliases saveds)
+	     ;; Set parameters to their new values.  This can't fail.
+	     ,@(map (lambda (p t) `(,p ,t #f #t)) param-aliases temps)
+	     ;; Remember we already converted (only call converters once!)
+	     (##core#set! ,convert? #f) ) )
+	   (##core#lambda () ,@body)
+	   (##core#lambda ()
+	    ;; Restore parameters to their original, saved values
+	    ,@(map (lambda (p s) `(,p ,s #f #t))
+		   param-aliases saveds) )) ) ) ) ) )))
 
 (##sys#extend-macro-environment
  'when '()
@@ -325,35 +334,22 @@
  'set!-values '()
  (##sys#er-transformer
   (lambda (form r c)
-    (##sys#check-syntax 'set!-values form '(_ #(variable 0) _))
-    (let ((vars (cadr form))
-	  (exp (caddr form)))
-      (cond ((null? vars)
-	     ;; may this be simply "exp"?
-	     `(##sys#call-with-values
-	       (##core#lambda () ,exp)
-	       (##core#lambda () (##core#undefined))) )
-	    ((null? (cdr vars))
-	     `(##core#set! ,(car vars) ,exp)) 
-	    (else
-	     (let ([aliases (map gensym vars)])
-	       `(##sys#call-with-values
-		 (##core#lambda () ,exp)
-		 (##core#lambda ,aliases
-			   ,@(map (lambda (v a)
-				    `(##core#set! ,v ,a))
-				  vars aliases) ) ) ) ) ) ))))
+    (##sys#check-syntax 'set!-values form '(_ lambda-list _))
+    (##sys#expand-multiple-values-assignment (cadr form) (caddr form)))))
 
 (set! ##sys#define-values-definition
   (##sys#extend-macro-environment
    'define-values '()
    (##sys#er-transformer
     (lambda (form r c)
-      (##sys#check-syntax 'define-values form '(_ #(variable 0) _))
-      (for-each (lambda (nm)
-                  (let ((name (##sys#get nm '##core#macro-alias nm)))
-                    (##sys#register-export name (##sys#current-module))))
-                (cadr form))
+      (##sys#check-syntax 'define-values form '(_ lambda-list _))
+      (##sys#decompose-lambda-list
+       (cadr form)
+       (lambda (vars argc rest)
+         (for-each (lambda (nm)
+                     (let ((name (##sys#get nm '##core#macro-alias nm)))
+                       (##sys#register-export name (##sys#current-module))))
+                   vars)))
       `(,(r 'set!-values) ,@(cdr form))))))
 
 (##sys#extend-macro-environment
@@ -424,25 +420,25 @@
 	    `(,%let-values (,(car vbindings))
 			   ,(fold (cdr vbindings))) ) ) ))))
 
+;;XXX do we need letrec*-values ?
 (##sys#extend-macro-environment
  'letrec-values '()
  (##sys#er-transformer
   (lambda (form r c)
-    (##sys#check-syntax 'letrec-values form '(_ list . _))
+    (##sys#check-syntax 'letrec-values form '(_ #((lambda-list . _) 0) . _))
     (let ((vbindings (cadr form))
-	  (body (cddr form)))
-      (let* ([vars (apply ##sys#append (map (lambda (x) (car x)) vbindings))] 
-	     [aliases (map (lambda (v) (cons v (r (gensym v)))) vars)] 
-	     [lookup (lambda (v) (cdr (assq v aliases)))] )
-	`(##core#let
-	  ,(map (lambda (v) (##sys#list v '(##core#undefined))) vars)
-	  ,@(map (lambda (vb)
-		   `(##sys#call-with-values 
-		     (##core#lambda () ,(cadr vb))
-		     (##core#lambda ,(map lookup (car vb))
-			       ,@(map (lambda (v) `(##core#set! ,v ,(lookup v))) (car vb)) ) ) )
-		 vbindings)
-	  ,@body) ) ) ) ) )
+          (body (cddr form)))
+      (let ((vars  (map car vbindings))
+            (exprs (map cadr vbindings)))
+        `(##core#let
+          ,(map (lambda (v) (##sys#list v '(##core#undefined)))
+                (foldl (lambda (l v) ; flatten multi-value formals
+                         (##sys#append l (##sys#decompose-lambda-list
+					  v (lambda (a _ _) a))))
+                       '()
+                       vars))
+          ,@(map ##sys#expand-multiple-values-assignment vars exprs)
+          ,@body))))))
 
 (##sys#extend-macro-environment
  'nth-value 
@@ -491,6 +487,7 @@
 	      (cond [(not (pair? b)) `(##core#if ,b ,(fold bs2) #f)]
 		    [(null? (cdr b)) `(##core#if ,(car b) ,(fold bs2) #f)]
 		    [else
+		     (##sys#check-syntax 'and-let* b '(symbol _))
 		     (let ((var (car b)))
 		       `(##core#let ((,var ,(cadr b)))
 			 (##core#if ,var ,(fold bs2) #f) ) ) ] ) ) ) ) ) ) ) )
@@ -882,23 +879,26 @@
 	  (%memv (r 'memv))
 	  (%else (r 'else)))
       (define (parse-clause c)
-	(let* ([var (and (symbol? (car c)) (car c))]
-	       [kinds (if var (cadr c) (car c))]
-	       [body (if var (cddr c) (cdr c))] )
+	(let* ((var (and (symbol? (car c)) (car c)))
+	       (kinds (if var (cadr c) (car c)))
+	       (body (if var
+			 `(##core#let ((,var ,exvar)) ,@(cddr c))
+			 `(##core#let () ,@(cdr c)))))
 	  (if (null? kinds)
-	      `(,%else 
-		,(if var
-		     `(##core#let ([,var ,exvar]) ,@body)
-		     `(##core#let () ,@body) ) )
-	      `((,%and ,kvar ,@(map (lambda (k) `(,%memv (##core#quote ,k) ,kvar)) kinds))
-		,(if var
-		     `(##core#let ([,var ,exvar]) ,@body)
-		     `(##core#let () ,@body) ) ) ) ) )
+	      `(,%else ,body)
+	      `((,%and ,kvar ,@(map (lambda (k)
+				      `(,%memv (##core#quote ,k) ,kvar)) kinds))
+		,body ) ) ) )
       `(,(r 'handle-exceptions) ,exvar
-	(##core#let ([,kvar (,%and (##sys#structure? ,exvar (##core#quote condition) )
-				   (##sys#slot ,exvar 1))])
-		    (,(r 'cond) ,@(map parse-clause (cddr form))
-		     (,%else (##sys#signal ,exvar)) ) )
+	(##core#let ((,kvar (,%and (##sys#structure? ,exvar
+						     (##core#quote condition))
+				   (##sys#slot ,exvar 1))))
+		    ,(let ((clauses (map parse-clause (cddr form))))
+		       `(,(r 'cond)
+			 ,@clauses
+			 ,@(if (assq %else clauses)
+			       `()   ; Don't generate two else clauses
+			       `((,%else (##sys#signal ,exvar)))) )) )
 	,(cadr form))))))
 
 
@@ -1057,11 +1057,11 @@
     (##sys#check-syntax 'rec form '(_ _ . _))
     (let ((head (cadr form)))
       (if (pair? head)
-	  `(##core#letrec ((,(car head) 
-			    (##core#lambda ,(cdr head)
-					   ,@(cddr form))))
-			  ,(car head))
-	  `(##core#letrec ((,head ,@(cddr form))) ,head))))))
+	  `(##core#letrec* ((,(car head) 
+			     (##core#lambda ,(cdr head)
+					    ,@(cddr form))))
+			   ,(car head))
+	  `(##core#letrec* ((,head ,@(cddr form))) ,head))))))
 
 
 ;;; Definitions available at macroexpansion-time:
@@ -1084,6 +1084,13 @@
     (##sys#check-syntax 'use x '(_ . #(_ 0)))
     `(##core#require-extension ,(cdr x) #t))))
 
+(##sys#extend-macro-environment
+ 'use-for-syntax '()
+ (##sys#er-transformer
+  (lambda (x r c)
+    (##sys#check-syntax 'use-for-syntax x '(_ . #(_ 0)))
+    `(,(r 'require-extension-for-syntax) ,@(cdr x)))))
+
 
 ;;; compiler syntax
 
@@ -1092,9 +1099,6 @@
  (syntax-rules ()
    ((_ name)
     (##core#define-compiler-syntax name #f))
-   ((_ (name . llist) body ...)		; DEPRECATED
-    (define-compiler-syntax name
-      (##sys#er-transformer (lambda llist body ...) 'name)))
    ((_ name transformer)
     (##core#define-compiler-syntax name transformer))))
 
@@ -1138,10 +1142,11 @@
  'functor '()
  (##sys#er-transformer
   (lambda (x r c)
-    (##sys#check-syntax 'functor x '(_ (symbol . #((symbol _) 0)) _ . _))
+    (##sys#check-syntax 'functor x '(_ (symbol . #((_ _) 0)) _ . _))
     (let* ((x (##sys#strip-syntax x))
 	   (head (cadr x))
 	   (name (car head))
+	   (args (cdr head))
 	   (exps (caddr x))
 	   (body (cdddr x))
 	   (registration
@@ -1150,8 +1155,14 @@
 	      ',(map (lambda (arg)
 		       (let ((argname (car arg))
 			     (exps (##sys#validate-exports (cadr arg) 'functor)))
+			 (unless (or (symbol? argname)
+				     (and (list? argname)
+					  (= 2 (length argname))
+					  (symbol? (car argname))
+					  (symbol? (cadr argname))))
+			   (##sys#syntax-error-hook "invalid functor argument" name arg))
 			 (cons argname exps)))
-		     (cdr head))
+		     args)
 	      ',(##sys#validate-exports exps 'functor)
 	      ',body)))
       `(##core#module
@@ -1229,20 +1240,17 @@
 			(##sys#put! 
 			 gname '##compiler#local-specializations
 			 (##sys#append
+			  (##sys#get gname '##compiler#local-specializations '())
 			  (list
 			   (cons atypes
 				 (if (and rtypes (pair? rtypes))
 				     (list
 				      (map (cut ##compiler#check-and-validate-type 
-					     <>
-					     'define-specialization)
+						<>
+						'define-specialization)
 					   rtypes)
 				      spec)
-				     (list spec))))
-			  (or (##compiler#variable-mark 
-			       gname
-			       '##compiler#local-specializations)
-			      '())))
+				     (list spec))))))
 			`(##core#begin
 			  (##core#declare (inline ,alias) (hide ,alias))
 			  (,%define (,alias ,@anames)
