@@ -1,6 +1,6 @@
 ;;;; compiler-syntax.scm - compiler syntax used internally
 ;
-; Copyright (c) 2009-2014, The Chicken Team
+; Copyright (c) 2009-2016, The CHICKEN Team
 ; All rights reserved.
 ;
 ; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
@@ -109,28 +109,26 @@
     (if (and (memq 'map standard-bindings) ; s.a.
 	     (> (length+ x) 2))
 	(let ((vars (map (lambda _ (gensym)) lsts)))
-	  `(,%let ((,%result (,%quote ()))
-		   (,%node #f)
-		   (,%proc ,(cadr x))
-		   ,@(map list vars lsts))		   
-		  ,@(map (lambda (var)
-			   `(##core#check (##sys#check-list ,var (,%quote map))))
-			 vars)
-		  (,%let ,%loop ,(map list vars vars)
-			 (,%if (,%and ,@(map (lambda (v) `(,%pair? ,v)) vars))
-			       (,%let ((,%res
-					(,%cons
-					 (,%proc
-					  ,@(map (lambda (v) `(##sys#slot ,v 0)) vars))
-					 (,%quote ()))))
-				      (,%if ,%node
-					    (##sys#setslot ,%node 1 ,%res)
-					    (,%set! ,%result ,%res))
-				      (,%set! ,%node ,%res)
-				      (##core#app
-				       ,%loop
-				       ,@(map (lambda (v) `(##sys#slot ,v 1)) vars)))
-			       ,%result))))
+	  `(,%let ((,%node (,%cons (##core#undefined) (,%quote ()))))
+	     (,%let ((,%result ,%node)
+		     (,%proc ,(cadr x))
+		     ,@(map list vars lsts))
+		    ,@(map (lambda (var)
+			     `(##core#check (##sys#check-list ,var (,%quote map))))
+			   vars)
+	       (,%let ,%loop ,(map list vars vars)
+		 (,%if (,%and ,@(map (lambda (v) `(,%pair? ,v)) vars))
+		       (,%let ((,%res
+				(,%cons
+				 (,%proc
+				  ,@(map (lambda (v) `(##sys#slot ,v 0)) vars))
+				 (,%quote ()))))
+			      (##sys#setslot ,%node 1 ,%res)
+			      (,%set! ,%node ,%res)
+			      (##core#app
+			       ,%loop
+			       ,@(map (lambda (v) `(##sys#slot ,v 1)) vars)))
+		       (##sys#slot ,%result 1))))))
 	x)))
 
 (define-internal-compiler-syntax ((o #%o) x r c) ()
