@@ -1,6 +1,6 @@
 ;;;; expand.scm - The HI/LO expander
 ;
-; Copyright (c) 2008-2014, The Chicken Team
+; Copyright (c) 2008-2016, The CHICKEN Team
 ; All rights reserved.
 ;
 ; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
@@ -99,7 +99,7 @@
 		ua))
 	alias) ) )
 
-(define (##sys#strip-syntax exp)
+(define (strip-syntax exp)
  ;; if se is given, retain bound vars
  (let ((seen '()))
    (let walk ((x exp))
@@ -125,7 +125,7 @@
                 (##sys#setslot vec i (walk (##sys#slot x i))))))
            (else x)))))
 
-(define strip-syntax ##sys#strip-syntax)
+(define ##sys#strip-syntax strip-syntax)
 
 (define (##sys#extend-se se vars #!optional (aliases (map gensym vars)))
   (for-each
@@ -306,14 +306,14 @@
 
 ;;; User-level macroexpansion
 
-(define (##sys#expand exp #!optional (se (##sys#current-environment)) cs?)
+(define (expand exp #!optional (se (##sys#current-environment)) cs?)
   (let loop ((exp exp))
     (let-values (((exp2 m) (##sys#expand-0 exp se cs?)))
       (if m
 	  (loop exp2)
 	  exp2) ) ) )
 
-(define expand ##sys#expand)
+(define ##sys#expand expand)
 
 
 ;;; Extended (DSSSL-style) lambda lists
@@ -438,6 +438,26 @@
    "redefinition of currently used defining form" ; help me find something better
    form))
 
+;;; Expansion of multiple values assignments.
+;
+; Given a lambda list and a multi-valued expression, returns a form that
+; will `set!` each variable to its corresponding value in order.
+
+(define (##sys#expand-multiple-values-assignment formals expr)
+  (##sys#decompose-lambda-list
+   formals
+   (lambda (vars argc rest)
+     (let ((aliases    (if (symbol? formals) '() (map gensym formals)))
+	   (rest-alias (if (not rest) '() (gensym rest))))
+       `(##sys#call-with-values
+	 (##core#lambda () ,expr)
+	 (##core#lambda
+	  ,(append aliases rest-alias)
+	  ,@(map (lambda (v a) `(##core#set! ,v ,a)) vars aliases)
+	  ,@(cond
+	      ((null? formals) '((##core#undefined)))
+	      ((null? rest-alias) '())
+	      (else `((##core#set! ,rest ,rest-alias))))))))))
 
 ;;; Expansion of bodies (and internal definitions)
 ;
@@ -457,14 +477,14 @@
 	      ((define-syntax) (if f (eq? f ##sys#define-syntax-definition) (eq? s id)))
 	      ((define-values) (if f (eq? f ##sys#define-values-definition) (eq? s id)))
 	      (else (eq? s id))))))
-    (define (fini vars vals mvars mvals body)
+    (define (fini vars vals mvars body)
       (if (and (null? vars) (null? mvars))
 	  (let loop ([body2 body] [exps '()])
 	    (if (not (pair? body2)) 
 		(cons 
 		 '##core#begin
 		 body) ; no more defines, otherwise we would have called `expand'
-		(let ([x (car body2)])
+		(let ((x (car body2)))
 		  (if (and (pair? x) 
 			   (let ((d (car x)))
 			     (and (symbol? d)
@@ -474,30 +494,32 @@
 		       '##core#begin
 		       (##sys#append (reverse exps) (list (expand body2))))
 		      (loop (cdr body2) (cons x exps)) ) ) ) )
-	  (let* ((vars (reverse vars))
-		 (result 
+	  (let* ((result
 		  `(##core#let
 		    ,(##sys#map
-		      (lambda (v) (##sys#list v (##sys#list '##core#undefined))) 
-		      (apply ##sys#append vars mvars) )
-		    ,@(map (lambda (v x) `(##core#set! ,v ,x)) vars (reverse vals))
-		    ,@(map (lambda (vs x)
-			     (let ([tmps (##sys#map gensym vs)])
-			       `(##sys#call-with-values
-				 (##core#lambda () ,x)
-				 (##core#lambda 
-				  ,tmps 
-				  ,@(map (lambda (v t)
-					   `(##core#set! ,v ,t)) 
-					 vs tmps) ) ) ) ) 
-			   (reverse mvars)
-			   (reverse mvals) )
+		      (lambda (v) (##sys#list v '(##core#undefined)))
+		      ;; vars are all normalised to lambda-lists: flatten them
+		      (foldl (lambda (l v)
+			       (##sys#append l (##sys#decompose-lambda-list
+						v (lambda (a _ _) a))))
+			     '()
+			     (reverse vars))) ; not strictly necessary...
+		    ,@(map (lambda (var val is-mvar?)
+			     ;; Non-mvars should expand to set! for
+			     ;; efficiency, but also because they must be
+			     ;; implicit multi-value continuations.
+			     (if is-mvar?
+				 (##sys#expand-multiple-values-assignment var val)
+				 `(##core#set! ,(car var) ,val)))
+			   (reverse vars)
+			   (reverse vals)
+			   (reverse mvars))
 		    ,@body) ) )
 	    (dd `(BODY: ,result))
 	    result)))
-    (define (fini/syntax vars vals mvars mvals body)
+    (define (fini/syntax vars vals mvars body)
       (fini
-       vars vals mvars mvals
+       vars vals mvars
        (let loop ((body body) (defs '()) (done #f))
 	 (cond (done `((##core#letrec-syntax
 			,(map cdr (reverse defs)) ,@body) ))
@@ -523,60 +545,62 @@
 		   #f)))
 	       (else (loop body defs #t))))))
     (define (expand body)
-      (let loop ([body body] [vars '()] [vals '()] [mvars '()] [mvals '()])
+      ;; Each #t in "mvars" indicates an MV-capable "var".  Non-MV
+      ;; vars (#f in mvars) are 1-element lambda-lists for simplicity.
+      (let loop ((body body) (vars '()) (vals '()) (mvars '()))
 	(if (not (pair? body))
-	    (fini vars vals mvars mvals body)
+	    (fini vars vals mvars body)
 	    (let* ((x (car body))
 		   (rest (cdr body))
 		   (exp1 (and (pair? x) (car x)))
 		   (head (and exp1 (symbol? exp1) exp1)))
 	      (if (not (symbol? head))
-		  (fini vars vals mvars mvals body)
+		  (fini vars vals mvars body)
 		  (cond
 		   ((comp 'define head)
 		     (##sys#check-syntax 'define x '(_ _ . #(_ 0)) #f se)
-		     (let loop2 ([x x])
-		       (let ([head (cadr x)])
-			 (cond [(not (pair? head))
+		     (let loop2 ((x x))
+		       (let ((head (cadr x)))
+			 (cond ((not (pair? head))
 				(##sys#check-syntax 'define x '(_ variable . #(_ 0)) #f se)
 				(when (eq? (car x) head) ; see above
 				  (##sys#defjam-error x))
-				(loop rest (cons head vars)
+				(loop rest (cons (list head) vars)
 				      (cons (if (pair? (cddr x))
 						(caddr x)
 						'(##core#undefined) )
 					    vals)
-				      mvars mvals) ]
-			       [(pair? (car head))
+				      (cons #f mvars)))
+			       ((pair? (car head))
 				(##sys#check-syntax
 				 'define x '(_ (_ . lambda-list) . #(_ 1)) #f se)
 				(loop2
-				 (##sys#expand-curried-define head (cddr x) se)) ]
-			       [else
+				 (##sys#expand-curried-define head (cddr x) se)))
+			       (else
 				(##sys#check-syntax
 				 'define x
 				 '(_ (variable . lambda-list) . #(_ 1)) #f se)
 				(loop rest
-				      (cons (car head) vars)
+				      (cons (list (car head)) vars)
 				      (cons `(##core#lambda ,(cdr head) ,@(cddr x)) vals)
-				      mvars mvals) ] ) ) ) )
+				      (cons #f mvars)))))))
 		    ((comp 'define-syntax head)
 		     (##sys#check-syntax 'define-syntax x '(_ _ . #(_ 1)) se)
-		     (fini/syntax vars vals mvars mvals body) )
+		     (fini/syntax vars vals mvars body))
 		    ((comp 'define-values head)
 		     ;;XXX check for any of the variables being `define-values'
-		     (##sys#check-syntax 'define-values x '(_ #(_ 0) _) #f se)
-		     (loop rest vars vals (cons (cadr x) mvars) (cons (caddr x) mvals)))
+		     (##sys#check-syntax 'define-values x '(_ lambda-list _) #f se)
+		     (loop rest (cons (cadr x) vars) (cons (caddr x) vals) (cons #t mvars)))
 		    ((comp '##core#begin head)
-		     (loop (##sys#append (cdr x) rest) vars vals mvars mvals) )
+		     (loop (##sys#append (cdr x) rest) vars vals mvars))
 		    (else
-		     (if (or (memq head vars) (memq head mvars))
-			 (fini vars vals mvars mvals body)
+		     (if (member (list head) vars)
+			 (fini vars vals mvars body)
 			 (let ((x2 (##sys#expand-0 x se cs?)))
 			   (if (eq? x x2)
-			       (fini vars vals mvars mvals body)
+			       (fini vars vals mvars body)
 			       (loop (cons x2 rest)
-				     vars vals mvars mvals) ) ) ) ) ) ) ) ) ) )
+				     vars vals mvars)))))))))))
     (expand body) ) )
 
 
@@ -619,10 +643,12 @@
 (define ##sys#syntax-error-culprit #f)
 (define ##sys#syntax-context '())
 
-(define (##sys#syntax-error-hook . args)
+(define (syntax-error . args)
   (apply ##sys#signal-hook #:syntax-error
 	 (##sys#strip-syntax args)))
 
+(define ##sys#syntax-error-hook syntax-error)
+
 (define ##sys#syntax-error/context
   (lambda (msg arg)
     (define (syntax-imports sym)
@@ -672,8 +698,6 @@
 			   (else (loop (cdr cx))))))))
 	  (##sys#syntax-error-hook (get-output-string out))))))
 
-(define syntax-error ##sys#syntax-error-hook)
-
 (define (##sys#syntax-rules-mismatch input)
   (##sys#syntax-error-hook "no rule matches form" input))
 
@@ -896,20 +920,15 @@
 	   ;; unhygienically this way.
 	   (mirror-rename (handler (rename form) rename compare)) ) ) )))
 
-(define (##sys#er-transformer handler) (make-er/ir-transformer handler #t))
-(define (##sys#ir-transformer handler) (make-er/ir-transformer handler #f))
+(define (er-macro-transformer handler) (make-er/ir-transformer handler #t))
+(define (ir-macro-transformer handler) (make-er/ir-transformer handler #f))
 
-(define er-macro-transformer ##sys#er-transformer)
-(define ir-macro-transformer ##sys#ir-transformer)
+(define ##sys#er-transformer er-macro-transformer)
+(define ##sys#ir-transformer ir-macro-transformer)
 
 
 ;;; Macro definitions:
 
-(define (##sys#mark-primitive prims)
-  (for-each
-   (lambda (a) (putp (cdr a) '##core#primitive (car a)))
-   prims))
-
 (##sys#extend-macro-environment
  'import '() 
  (##sys#er-transformer 
@@ -1163,9 +1182,12 @@
                          (and (pair? (car clause))
                               (c (r 'quote) (caar clause))))
 		     (expand rclauses (strip-syntax (car clause)))
-		     (if (null? (cdr clause))
-			 (car clause)
-			 `(##core#begin ,@(cdr clause))))
+		     (cond ((and (fx= (length clause) 3)
+				 (c %=> (cadr clause)))
+			    `(,(caddr clause) ,(car clause)))
+			   ((null? (cdr clause))
+			    (car clause))
+			   (else `(##core#begin ,@(cdr clause)))))
 		    ((null? (cdr clause)) 
 		     `(,%or ,(car clause) ,(expand rclauses #f)))
 		    ((and (fx= (length clause) 3)
@@ -1459,7 +1481,7 @@
 			     '(##core#undefined))))
 		     (else
 		      (##sys#check-syntax 
-		       'module x '(_ symbol _ (symbol . #(_ 1))))
+		       'module x '(_ symbol _ (symbol . #(_ 0))))
 		      (##sys#instantiate-functor
 		       name
 		       (car app)	; functor name
