@@ -1,6 +1,6 @@
 ;;;; irregex.scm - container for irregex-core.scm
 ;
-; Copyright (c) 2010-2014, The Chicken Team
+; Copyright (c) 2010-2016, The CHICKEN Team
 ; All rights reserved.
 ;
 ; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
@@ -89,6 +89,10 @@
 ;; so find-tail is unused (it's only used in the "find" definition)
 (declare (unused find-tail))
 
+;; This is to silence an "always true" warning that we can't fix
+;; because we don't want to needlessly change irregex-core.
+(declare (type (*allow-utf8-mode?* boolean)))
+
 (define-syntax build-cache
   (er-macro-transformer 
    (lambda (x r c)
@@ -232,12 +236,6 @@
        (vector-set! (##sys#slot m 1) (+ 4 (* t 2)) chunk)
        (vector-set! (##sys#slot m 1) (+ 5 (* t 2)) index)))))
 
-(declare (unused %irregex-error))
-(define-compiler-syntax %irregex-error
-  (syntax-rules ()
-    ((_ args ...)
-     (error args ...))))
-
 (include "irregex-core.scm")
 (include "irregex-utils.scm")
 
