@@ -1,6 +1,6 @@
 ;;;; lolevel.import.scm - import library for "lolevel" module
 ;
-; Copyright (c) 2008-2014, The Chicken Team
+; Copyright (c) 2008-2016, The CHICKEN Team
 ; All rights reserved.
 ;
 ; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
@@ -44,7 +44,6 @@
    make-weak-locative
    move-memory!
    mutate-procedure!
-   null-pointer
    number-of-bytes
    number-of-slots
    object->pointer
@@ -93,4 +92,5 @@
    record-instance-type
    set-procedure-data!
    tag-pointer
-   tagged-pointer?))
+   tagged-pointer?
+   vector-like?))
