@@ -1,6 +1,6 @@
 ;;;; posix-common.scm - common code for UNIX and Windows versions of the posix unit
 ;
-; Copyright (c) 2010-2014, The Chicken Team
+; Copyright (c) 2010-2016, The CHICKEN Team
 ; All rights reserved.
 ;
 ; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
@@ -96,6 +96,11 @@ static char C_time_string [TIME_STRING_MAXLENGTH + 1];
 #define C_readdir(h,e)      C_set_block_item(e, 0, (C_word) readdir((DIR *)C_block_item(h, 0)))
 #define C_foundfile(e,b,l)    (C_strlcpy(C_c_string(b), ((struct dirent *) C_block_item(e, 0))->d_name, l), C_fix(strlen(((struct dirent *) C_block_item(e, 0))->d_name)))
 
+/* It is assumed that 'int' is-a 'long' */
+#define C_ftell(p)          C_fix(ftell(C_port_file(p)))
+#define C_fseek(p, n, w)    C_mk_nbool(fseek(C_port_file(p), C_num_to_int(n), C_unfix(w)))
+#define C_lseek(fd, o, w)     C_fix(lseek(C_unfix(fd), C_unfix(o), C_unfix(w)))
+
 #ifdef HAVE_SETENV
 # define C_unsetenv(s)      (unsetenv((char *)C_data_pointer(s)), C_SCHEME_TRUE)
 # define C_setenv(x, y)     C_fix(setenv((char *)C_data_pointer(x), (char *)C_data_pointer(y), 1))
@@ -216,10 +221,11 @@ EOF
 
 (define (##sys#stat file link err loc)
   (let ((r (cond ((fixnum? file) (##core#inline "C_fstat" file))
+                 ((port? file) (##core#inline "C_fstat" (port->fileno file)))
                  ((string? file)
                   (let ((path (##sys#make-c-string
 			       (##sys#platform-fixup-pathname
-				(##sys#expand-home-path file))
+                                file)
 			       loc)))
 		    (if link
 			(##core#inline "C_lstat" path)
@@ -248,7 +254,7 @@ EOF
    (lambda (f t)
      (##sys#check-number t 'set-file-modification-time)
      (let ((r ((foreign-lambda int "set_file_mtime" c-string scheme-object)
-	       (##sys#expand-home-path f) t)))
+	       f t)))
        (when (fx< r 0)
 	 (posix-error 
 	  #:file-error 'set-file-modification-time
@@ -295,6 +301,48 @@ EOF
   (eq? 'directory (file-type file #f #f)))
 
 
+;;; File position access:
+
+(define-foreign-variable _seek_set int "SEEK_SET")
+(define-foreign-variable _seek_cur int "SEEK_CUR")
+(define-foreign-variable _seek_end int "SEEK_END")
+
+(define seek/set _seek_set)
+(define seek/end _seek_end)
+(define seek/cur _seek_cur)
+
+(define set-file-position!
+  (lambda (port pos . whence)
+    (let ((whence (if (pair? whence) (car whence) _seek_set)))
+      (##sys#check-exact pos 'set-file-position!)
+      (##sys#check-exact whence 'set-file-position!)
+      (unless (cond ((port? port)
+		     (and (eq? (##sys#slot port 7) 'stream)
+			  (##core#inline "C_fseek" port pos whence) ) )
+		    ((fixnum? port)
+		     (##core#inline "C_lseek" port pos whence))
+		    (else
+		     (##sys#signal-hook #:type-error 'set-file-position! "invalid file" port)) )
+	(posix-error #:file-error 'set-file-position! "cannot set file position" port pos) ) ) ) )
+
+(define file-position
+  (getter-with-setter
+   (lambda (port)
+     (let ((pos (cond ((port? port)
+		       (if (eq? (##sys#slot port 7) 'stream)
+			   (##core#inline "C_ftell" port)
+			   -1) )
+		      ((fixnum? port)
+		       (##core#inline "C_lseek" port 0 _seek_cur) )
+		      (else
+		       (##sys#signal-hook #:type-error 'file-position "invalid file" port)) ) ) )
+       (when (< pos 0)
+	 (posix-error #:file-error 'file-position "cannot retrieve file position of port" port) )
+       pos) )
+   set-file-position!		; doesn't accept WHENCE
+   "(file-position port)"))
+
+
 ;;; Using file-descriptors:
 
 (define-foreign-variable _stdin_fileno int "STDIN_FILENO")
@@ -382,21 +430,38 @@ EOF
 	(unless (fx= 0 (##core#inline "C_rmdir" sname))
 	  (posix-error #:file-error 'delete-directory "cannot delete directory" dir) )))
     (##sys#check-string name 'delete-directory)
-    (let ((name (##sys#expand-home-path name)))
-      (if recursive
-	  (let ((files (find-files ; relies on `find-files' to list dir-contents before dir
-			name 
-			dotfiles: #t
-			follow-symlinks: #f)))
-	    (for-each
-	     (lambda (f)
-	       ((cond ((symbolic-link? f) delete-file)
-		      ((directory? f) rmdir)
-		      (else delete-file))
-		f))
-	     files)
-	    (rmdir name))
-	  (rmdir name)))))
+    (if recursive
+      (let ((files (find-files ; relies on `find-files' to list dir-contents before dir
+                     name
+                     dotfiles: #t
+                     follow-symlinks: #f)))
+        (for-each
+          (lambda (f)
+            ((cond ((symbolic-link? f) delete-file)
+                   ((directory? f) rmdir)
+                   (else delete-file))
+             f))
+          files)
+        (rmdir name))
+      (rmdir name))))
+
+(define-inline (*create-directory loc name)
+  (unless (fx= 0 (##core#inline "C_mkdir" (##sys#make-c-string name loc)))
+    (posix-error #:file-error loc "cannot create directory" name)) )
+
+(define create-directory
+  (lambda (name #!optional parents?)
+    (##sys#check-string name 'create-directory)
+    (unless (or (fx= 0 (##sys#size name))
+                (file-exists? name))
+      (if parents?
+        (let loop ((dir (let-values (((dir file ext) (decompose-pathname name)))
+                          (if file (make-pathname dir file ext) dir))))
+          (when (and dir (not (directory? dir)))
+            (loop (pathname-directory dir))
+            (*create-directory 'create-directory dir)) )
+        (*create-directory 'create-directory name) ) )
+    name))
 
 (define directory
   (lambda (#!optional (spec (current-directory)) show-dotfiles?)
@@ -406,7 +471,7 @@ EOF
 	  [entry (##sys#make-pointer)] )
       (##core#inline 
        "C_opendir"
-       (##sys#make-c-string (##sys#expand-home-path spec) 'directory) handle)
+       (##sys#make-c-string spec 'directory) handle)
       (if (##sys#null-pointer? handle)
 	  (posix-error #:file-error 'directory "cannot open directory" spec)
 	  (let loop ()
@@ -426,7 +491,6 @@ EOF
 		      (loop)
 		      (cons file (loop)) ) ) ) ) ) ) ) )
 
-
 ;;; Filename globbing:
 
 (define glob
@@ -449,37 +513,40 @@ EOF
 
 ;;; Find matching files:
 
-(define ##sys#find-files
-  (lambda (dir pred action id limit follow dot loc)
-    (##sys#check-string dir loc)
-    (let* ((depth 0)
-	   (lproc
-	    (cond ((not limit) (lambda _ #t))
-		  ((fixnum? limit) (lambda _ (fx< depth limit)))
-		  (else limit) ) )
-	   (pproc
-	    (if (procedure? pred)
-		pred
-		(let ((pred (irregex pred))) ; force compilation
-		  (lambda (x) (irregex-match pred x))) ) ) )
-      (let loop ((fs (glob (make-pathname dir (if dot "?*" "*"))))
-		 (r id) )
-	(if (null? fs)
-	    r
-	    (let ((f (##sys#slot fs 0))
-		  (rest (##sys#slot fs 1)) )
-	      (cond ((directory? f)
-		     (cond ((member (pathname-file f) '("." "..")) (loop rest r))
-			   ((and (symbolic-link? f) (not follow))
-			    (loop rest (if (pproc f) (action f r) r)))
-			   ((lproc f)
-			    (loop rest
-				  (fluid-let ((depth (fx+ depth 1)))
-				    (loop (glob (make-pathname f (if dot "?*" "*")))
-					  (if (pproc f) (action f r) r)) ) ) )
-			   (else (loop rest (if (pproc f) (action f r) r))) ) )
-		    ((pproc f) (loop rest (action f r)))
-		    (else (loop rest r)) ) ) ) ) ) ) )
+(define (##sys#find-files dir pred action id limit follow dot loc)
+  (##sys#check-string dir loc)
+  (let* ((depth 0)
+         (lproc
+          (cond ((not limit) (lambda _ #t))
+                ((fixnum? limit) (lambda _ (fx< depth limit)))
+                (else limit) ) )
+         (pproc
+          (if (procedure? pred)
+              pred
+              (let ((pred (irregex pred))) ; force compilation
+                (lambda (x) (irregex-match pred x))))))
+    (let loop ((dir dir)
+               (fs (directory dir dot))
+               (r id))
+      (if (null? fs)
+          r
+          (let* ((filename (##sys#slot fs 0))
+                 (f (make-pathname dir filename))
+                 (rest (##sys#slot fs 1)))
+            (cond ((directory? f)
+                   (cond ((member filename '("." "..")) (loop dir rest r))
+                         ((and (symbolic-link? f) (not follow))
+                          (loop dir rest (if (pproc f) (action f r) r)))
+                         ((lproc f)
+                          (loop dir
+                                rest
+                                (fluid-let ((depth (fx+ depth 1)))
+                                  (loop f
+                                        (directory f dot)
+                                        (if (pproc f) (action f r) r)))))
+                         (else (loop dir rest (if (pproc f) (action f r) r)))))
+                  ((pproc f) (loop dir rest (action f r)))
+                  (else (loop dir rest r))))))))
 
 (define (find-files dir #!key (test (lambda _ #t))
 			      (action (lambda (x y) (cons x y)))
