@@ -1,6 +1,6 @@
 ; scheduler.scm - Basic scheduler for multithreading
 ;
-; Copyright (c) 2008-2014, The Chicken Team
+; Copyright (c) 2008-2016, The CHICKEN Team
 ; Copyright (c) 2000-2007, Felix L. Winkelmann
 ; All rights reserved.
 ;
@@ -30,10 +30,12 @@
   (disable-interrupts)
   (hide ready-queue-head ready-queue-tail ##sys#timeout-list
 	##sys#update-thread-state-buffer ##sys#restore-thread-state-buffer
-	remove-from-ready-queue ##sys#unblock-threads-for-i/o ##sys#force-primordial
+	remove-from-ready-queue ##sys#unblock-threads-for-i/o
+	;; This isn't hidden ATM to allow set!ing it as a hook/workaround
+	; ##sys#force-primordial
 	fdset-set fdset-test create-fdset stderr
 	##sys#clear-i/o-state-for-thread! ##sys#abandon-mutexes) 
-  (not inline ##sys#interrupt-hook)
+  (not inline ##sys#interrupt-hook ##sys#force-primordial)
   (unsafe)
   (foreign-declare #<<EOF
 #ifdef HAVE_ERRNO_H
@@ -66,7 +68,33 @@ C_word C_msleep(C_word ms) {
 }
 #endif
 
-#ifdef HAVE_POSIX_POLL
+#ifdef NO_POSIX_POLL
+
+/* Shouldn't we include <sys/select.h> here? */
+static fd_set C_fdset_input, C_fdset_output;
+
+#define C_fd_input_ready(fd,pos)  C_mk_bool(FD_ISSET(C_unfix(fd), &C_fdset_input))
+#define C_fd_output_ready(fd,pos)  C_mk_bool(FD_ISSET(C_unfix(fd), &C_fdset_output))
+
+C_inline int C_ready_fds_timeout(int to, double tm) {
+  struct timeval timeout;
+  timeout.tv_sec = tm / 1000;
+  timeout.tv_usec = fmod(tm, 1000) * 1000;
+  /* we use FD_SETSIZE, but really should use max fd */
+  return select(FD_SETSIZE, &C_fdset_input, &C_fdset_output, NULL, to ? &timeout : NULL);
+}
+
+C_inline void C_prepare_fdset(int length) {
+  FD_ZERO(&C_fdset_input);
+  FD_ZERO(&C_fdset_output);
+}
+
+C_inline void C_fdset_add(int fd, int input, int output) {
+  if (input) FD_SET(fd, &C_fdset_input);
+  if (output) FD_SET(fd, &C_fdset_output);
+}
+
+#else
 #  include <poll.h>
 #  include <assert.h>
 
@@ -98,32 +126,6 @@ C_inline void C_fdset_add(int fd, int input, int output) {
   C_fdset_set[C_fdset_nfds].events = ((input ? POLLIN : 0) | (output ? POLLOUT : 0));
   C_fdset_set[C_fdset_nfds++].fd = fd;
 }
-
-#else
-
-/* Shouldn't we include <sys/select.h> here? */
-static fd_set C_fdset_input, C_fdset_output;
-
-#define C_fd_input_ready(fd,pos)  C_mk_bool(FD_ISSET(C_unfix(fd), &C_fdset_input))
-#define C_fd_output_ready(fd,pos)  C_mk_bool(FD_ISSET(C_unfix(fd), &C_fdset_output))
-
-C_inline int C_ready_fds_timeout(int to, double tm) {
-  struct timeval timeout;
-  timeout.tv_sec = tm / 1000;
-  timeout.tv_usec = fmod(tm, 1000) * 1000;
-  /* we use FD_SETSIZE, but really should use max fd */
-  return select(FD_SETSIZE, &C_fdset_input, &C_fdset_output, NULL, to ? &timeout : NULL);
-}
-
-C_inline void C_prepare_fdset(int length) {
-  FD_ZERO(&C_fdset_input);
-  FD_ZERO(&C_fdset_output);
-}
-
-C_inline void C_fdset_add(int fd, int input, int output) {
-  if (input) FD_SET(fd, &C_fdset_input);
-  if (output) FD_SET(fd, &C_fdset_output);
-}
 #endif
 EOF
 ) )
@@ -154,6 +156,7 @@ EOF
     (set! ##sys#current-thread thread)
     (##sys#setslot thread 3 'running)
     (##sys#restore-thread-state-buffer thread)
+    ;;XXX WRONG! this sets the t/i-period ("quantum") for the _next_ thread
     (##core#inline "C_set_initial_timer_interrupt_period" (##sys#slot thread 9))
     ((##sys#slot thread 1)) )
   (let* ([ct ##sys#current-thread]
@@ -386,26 +389,33 @@ EOF
   ((foreign-lambda void "C_prepare_fdset" int) (##sys#length ##sys#fd-list))
   (let loop ((lst ##sys#fd-list))
     (unless (null? lst)
-      (let ((fd (caar lst)))
+      (let ((fd (caar lst))
+	    (input #f)
+	    (output #f))
 	(for-each
 	 (lambda (t)
 	   (let ((p (##sys#slot t 11)))
-             (when (pair? p) ; (FD . RWFLAGS)? (can also be mutex or thread)
-               (fdset-set fd (cdr p)))))
+	     ;; XXX: This should never be false, because otherwise the
+	     ;; thread is not supposed to be on ##sys#fd-list!
+	     (when (pair? p) ; (FD . RWFLAGS)? (can also be mutex or thread)
+	       (let ((i/o (cdr p)))
+		 (case i/o
+		   ((#t #:input)
+		    (set! input #t))
+		   ((#f #:output)
+		    (set! output #t))
+		   ((#:all)
+		    (set! input #t)
+		    (set! output #t))
+		   (else
+		    (panic
+		     (sprintf "create-fdset: invalid i/o direction: ~S (fd = ~S)" i/o fd))))))))
 	 (cdar lst))
+	;; Our position in fd-list must match fdset array position, so
+	;; always add an fdset entry, even if input & output are #f.
+	((foreign-lambda void "C_fdset_add" int bool bool) fd input output)
 	(loop (cdr lst))))))
 
-(define (fdset-set fd i/o)
-  (let ((fdset-add! (foreign-lambda void "C_fdset_add" int bool bool)))
-    (dbg "setting fdset for " fd " to " i/o)
-    (case i/o
-      ((#t #:input) (fdset-add! fd #t #f))
-      ((#f #:output) (fdset-add! fd #f #t))
-      ((#:all) (fdset-add! fd #t #t))
-      (else
-       (panic
-        (sprintf "fdset-set: invalid i/o direction: ~S (fd = ~S)" i/o fd))))))
-
 (define (fdset-test inf outf i/o)
   (case i/o
     ((#t #:input) inf)
