@@ -508,6 +508,52 @@ A
 
 (assert (= 2 guard-called))
 
+;; Parameters are reset correctly (#1227, pointed out by Joo ChurlSoo)
+
+(let ((a (make-parameter 1 number->string))
+      (b (make-parameter 2 number->string)))
+  (assert (equal? (list "1" "2") (list (a) (b))))
+
+  (assert (equal? (list "10" "20")
+		  (parameterize ((a 10) (b 20)) (list (a) (b)))))
+
+  (assert (equal? (list "1" "2") (list (a) (b))))
+
+  (handle-exceptions exn #f (parameterize ((a 10) (b 'x)) (void)))
+
+  (assert (equal? (list "1" "2") (list (a) (b))))
+
+  (parameterize ((a 10) (b 30) (a 20))
+    (assert (equal? (list "20" "30") (list (a) (b)))))
+
+  (assert (equal? (list "1" "2") (list (a) (b)))) )
+
+;; Special-cased parameters are reset correctly (#1285, regression
+;; caused by fix for #1227)
+
+(let ((original-input (current-input-port))
+      (original-output (current-output-port))
+      (original-error (current-error-port))
+      (original-exception-handler (current-exception-handler)))
+  (call-with-output-string
+   (lambda (out)
+     (call-with-input-string
+      "foo"
+      (lambda (in)
+	(parameterize ((current-output-port out)
+		       (current-error-port out)
+		       (current-input-port in)
+		       (current-exception-handler list))
+	  (display "bar")
+	  (display "!" (current-error-port))
+	  (assert (equal? (read) 'foo))
+	  (assert (equal? (get-output-string out) "bar!"))
+	  (assert (equal? (signal 'baz) '(baz))))))))
+  (assert (equal? original-input (current-input-port)))
+  (assert (equal? original-output (current-output-port)))
+  (assert (equal? original-error (current-error-port)))
+  (assert (equal? original-exception-handler (current-exception-handler))))
+
 ;;; vector and blob limits
 
 (assert-fail (make-blob -1))
