@@ -64,7 +64,7 @@ TYPESDB=../types.db
 cp $TYPESDB test-repository/types.db
 
 compile="../csc -types ${TYPESDB} -ignore-repository ${COMPILE_OPTIONS} -o a.out"
-compile2="../csc -compiler $CHICKEN -v -I${TEST_DIR}/.. -L${TEST_DIR}.. -include-path ${TEST_DIR}/.."
+compile2="../csc -compiler $CHICKEN -v -I${TEST_DIR}/.. -L${TEST_DIR}/.. -include-path ${TEST_DIR}/.."
 compile_s="../csc -s -types ${TYPESDB} -ignore-repository ${COMPILE_OPTIONS} -v -I${TEST_DIR}/.. -L${TEST_DIR}/.. -include-path ${TEST_DIR}/.."
 interpret="../csi -n -include-path ${TEST_DIR}/.."
 
@@ -113,7 +113,7 @@ rm -f foo.types foo.import.*
 $compile specialization-test-1.scm -emit-type-file foo.types -specialize \
   -debug ox -emit-import-library foo
 ./a.out
-$compile specialization-test-2.scm -types foo.types -specialize -debug ox
+$compile specialization-test-2.scm -types foo.types -types specialization-test-2.types -feature chicken-bootstrap -specialize -debug ox
 ./a.out
 rm -f foo.types foo.import.*
 
@@ -165,7 +165,7 @@ echo "*** Skipping \"feeley-dynwind\" for now ***"
 
 echo "======================================== lolevel tests ..."
 $interpret -s lolevel-tests.scm
-$compile lolevel-tests.scm
+$compile -specialize lolevel-tests.scm
 ./a.out
 
 echo "======================================== arithmetic tests ..."
@@ -223,6 +223,9 @@ $compile -s square-functor.import.scm
 $interpret -bnq use-square-functor.scm
 $compile use-square-functor.scm
 ./a.out
+$compile -s use-square-functor.scm -J
+$interpret -nqe '(import sf1)' -e '(import sf2)'
+rm -f sf1.import.* sf2.import.* lst.import.* mod.import.*
 
 echo "======================================== compiler syntax tests ..."
 $compile compiler-syntax-tests.scm
@@ -341,6 +344,9 @@ rm -fr tmpdir
 mkdir tmpdir
 touch tmpdir/.dotfile
 
+echo "======================================== find-files tests ..."
+$interpret -bnq test-find-files.scm
+
 if test -z "$MSYSTEM"; then
     ln -s /usr tmpdir/symlink
 fi
@@ -361,6 +367,13 @@ for s in 100000 120000 200000 250000 300000 350000 400000 450000 500000; do
     ../chicken -ignore-repository ../utils.scm -:s$s -output-file tmp.c -include-path ${TEST_DIR}/..
 done
 
+echo "======================================== heap literal stress test ..."
+$compile heap-literal-stress-test.scm
+for s in 100000 120000 200000 250000 300000 350000 400000 450000 500000; do
+  echo "  $s"
+  ./a.out -:d -:g -:hi$s
+done
+
 echo "======================================== symbol-GC tests ..."
 $compile symbolgc-tests.scm
 # Currently disabled, because this may leave 1 symbol unreclaimed.
