@@ -6,6 +6,10 @@
       (let ((x (+ 3 4)))
 	(if x 1 2)))))			; expected boolean but got number in conditional
 
+(define (b)
+  (let ((x #t))
+    (if x 1 2)))			; #t is always true
+
 (define (foo x)
   (if x 				; branches return differing number of results
       (values 1 2)
@@ -161,3 +165,45 @@
 ;; multiple-value return syntax
 (: mv (-> . *))
 (: mv (procedure () . *))
+
+;; procedures from the type environment should still enforce, etc.
+(let ((x (the (or fixnum string) _))
+      (f (the (forall (a)
+                (a -> (-> a)))
+              (lambda (a)
+                (lambda () a)))))
+  (((f +)) x)  ; (or fixnum string) -> fixnum
+  (fixnum? x)) ; should report
+
+;; typeset reduction
+
+(: char-or-string? (* -> boolean : (or char string)))
+
+(let ((x _))
+  (if (char-or-string? x)
+      (symbol? x)   ; should report with x = (or char string)
+      (string? x))) ; should not report
+
+(let ((x (the fixnum _)))
+  (if (char-or-string? x)
+      (symbol? x)   ; should report with x = (or char string)
+      (string? x))) ; should report with x = fixnum
+
+(let ((x (the (or char symbol) _)))
+  (if (char-or-string? x)
+      (symbol? x)   ; should report with x = char
+      (string? x))) ; should report with x = symbol
+
+(let ((x (the (or char symbol string) _)))
+  (if (char-or-string? x)
+      (symbol? x)   ; should report with x = (or char string)
+      (string? x))) ; should report with x = symbol
+
+;; list- and pair-type argument matching
+
+(let ((f (the (pair -> *) _))) (f (list)))        ; warning
+(let ((f (the (pair -> *) _))) (f (make-list x))) ; no warning
+(let ((f (the (null -> *) _))) (f (list 1)))      ; warning
+(let ((f (the (null -> *) _))) (f (make-list x))) ; no warning
+(let ((f (the (list -> *) _))) (f (cons 1 2)))    ; warning
+(let ((f (the (list -> *) _))) (f (cons 1 x)))    ; no warning
